# Example of how to search for services on the headset profile

# Register AG Service Record
proc RegisterAGServiceRecord {AGServerChannel} {	
  global UsingPrimitives
  set hs_record {0x09 0x00 0x01 \
                 0x35 0x06 \
                 0x19 0x11 0x12 \
                 0x19 0x12 0x03 \
                 0x09 0x00 0x04 \
                 0x35 0x0c \
                 0x35 0x03 \
                 0x19 0x01 0x00 \
                 0x35 0x05 \
                 0x19 0x00 0x03 \
                 0x08} 
  set hs_record [concat $hs_record $AGServerChannel]
  set hs_record [concat $hs_record {0x09 0x00 0x09 \
                                    0x35 0x08 \
                                    0x35 0x06 \
                                    0x19 0x11 0x12 \
                                    0x09 0x01 0x00 \
                                    0x09 0x01 0x00 \
                                    0x25 0x0d \
                                    0x56 0x6f 0x69 \
                                    0x63 0x65 0x20 \
                                    0x47 0x61 0x74 \
                                    0x65 0x77 0x61 0x79}]
  if {$UsingPrimitives} {
    SDS_REGISTER_REQ 0x8000 $hs_record [llength $hs_record]    
    set result [SDS_REGISTER_CFM]
  } else {
    set result [sds_register_req - $hs_record [llength $hs_record]]
  }
  set service_record_handle [lindex $result 1]
  set retval [lindex $result 2]
  puts "Service Record Handle: $service_record_handle"
  set retval [concat $retval $service_record_handle]
  puts "retval: $retval"
  return $retval
}

# Pair with Headset
proc PairWithHeadset {} {
  global bd_addr.lap bd_addr.uap bd_addr.nap
  global pin_code pin_length
  after 3000

  # Store a default PIN to use when pairing
  set pin_code {0x31 0x32 0x33 0x34}
  set pin_length 4

  # Remove the device we're going to pair with from the security manager's
  # settings
  dm_sm_remove_device_req ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap}

  # Set the security level
  set authentication_enabled 1
  set encryption_enabled 0
  dm_sm_set_sec_mode_req 3 0
}

# Request to Open SDC Search
proc StartSDPSearch {sel} {
  switch $sel {
    0 {SDC_OPEN_SEARCH_REQ - - - -}
    1 {set result [lindex [SDC_OPEN_SEARCH_CFM] 1]}
    2 {set result [lindex [sdc_open_search_req - - - -] 1]} 
  } 
  if {$sel > 0} {
    puts "Result SDC_OPEN_SEARCH_CFM: $result"
    # set the pair status depending on whether the pairing was a success
    if {$result == 0} {
      set pair_status 1
    } else {
      set pair_status 4
    }
  }
}

# Wait for Link Key Request and Reject It 
proc WaitLinkKeyRequestReject {} {
  # Wait for a link key request and reject it 
  # It doesn't matter what we send down as a link key
  DM_SM_LINK_KEY_REQUEST_IND
  DM_SM_LINK_KEY_REQUEST_RES - - - {0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0} 0
}

# Wait for PIN Code and Respond with Stored Value = 1234
proc WaitPinCodeRequest {} {
  global pin_code pin_length
  DM_SM_PIN_REQUEST_IND
  DM_SM_PIN_REQUEST_RES - - - $pin_length $pin_code
}

# Wait for Link Key
proc WaitLinkKeyRequest {} {
  global key
  DM_SM_LINK_KEY_IND

  # Store the link key in the security manager
  dm_sm_add_device_req - - - 1 1 1 $key
}

# Search for HID Service Record on the Headset
proc SearchHIDServiceRecord {} {
  global UsingPrimitives

  #search for HID service records because this should
  #give a failure as we are communicating with Headset
  if {$UsingPrimitives} {
    DC_SERVICE_SEARCH_REQ - - - - 5 {0x35 0x3 0x19 0x11 0x24} 1
    set result [SDC_SERVICE_SEARCH_CFM]
  } else {
    set result [sdc_service_search_req - - - - 5 {0x35 0x3 0x19 0x11 0x24} 1]
  }
  puts "Service Record Search Result HID: $result"
}

proc  SearchHeadsetServiceRecord {} {
  global UsingPrimitives

  #search for Headset service records
  if {$UsingPrimitives} {
    SDC_SERVICE_SEARCH_REQ - - - - 5 {0x35 0x3 0x19 0x11 0x08} 1
    set result [SDC_SERVICE_SEARCH_CFM]
  } else {
  set result [sdc_service_search_req - - - - 5 {0x35 0x3 0x19 0x11 0x08} 1]
  }
  puts "Service Record Search Result HS: $result"
  #Strip off the service record handle
  set srh [lindex $result 3]
  set retval0 [expr {0x01000000 * [lindex $srh 0]}]
  set retval1 [expr {0x010000 * [lindex $srh 1]}]
  set retval2 [expr {0x0100 * [lindex $srh 2]}]
  set retval3 [lindex $srh 3]
  set retval [expr {$retval0 + $retval1 + $retval2 + $retval3}]
  return $retval
}

# Search for Generic Audio Service on the Headset
proc SearchGenericAudioServiceRecord {} {
  global UsingPrimitives

  #search for Generic Audio service records
  if {$UsingPrimitives} {
    SDC_SERVICE_SEARCH_REQ - - - - 5 {0x35 0x3 0x19 0x12 0x03} 1
    set result [SDC_SERVICE_SEARCH_CFM]
  } else {
  set result [sdc_service_search_req - - - - 5 {0x35 0x3 0x19 0x12 0x03} 1]
  }
  puts "Service Record Search Result GA: $result"
}

# Search for Attributes on the Headset
proc ServiceAttributes {} {
  global UsingPrimitives
  global bd_addr.lap bd_addr.uap bd_addr.nap

  #Get Service Record handle for the Headset
  set srh [SearchHeadsetServiceRecord]
  puts [format "Service Record Handle: 0x%x" $srh]

  #Search for the ServiceClassIDList Attribute (Attribute ID = 0x0001) on Headset
  if {$UsingPrimitives} {
    SDC_SERVICE_ATTRIBUTE_REQ - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x00 0x01} 10
    set result [SDC_SERVICE_ATTRIBUTE_CFM]
  } else {
  set result [sdc_service_attribute_req - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x00 0x01} 10]
  }
  puts "Service Attribute Result for ServiceClassIDList: $result"

  #Search for the ProtocolDescriptorList Attribute (Attribute ID = 0x0004) on Headset
  if {$UsingPrimitives} {
    SDC_SERVICE_ATTRIBUTE_REQ - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x00 0x04} 10
    set result [SDC_SERVICE_ATTRIBUTE_CFM]
  } else {
    set result [sdc_service_attribute_req - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x00 0x04} 10]
  }
  puts "Service Attribute Result for ProtocolDescriptorList: $result"

  #Search for the BluetoothProfileDescriptorList Attribute (Attribute ID = 0x0009) on Headset
  if {$UsingPrimitives} {
    SDC_SERVICE_ATTRIBUTE_REQ - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x00 0x09} 10
    set result [SDC_SERVICE_ATTRIBUTE_CFM]
  } else {
    set result [sdc_service_attribute_req - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x00 0x09} 10]
  }
  puts "Service Attribute Result for BluetoothProfileDescriptorList: $result"
        
  #Search for the ServiceName Attribute (Attribute ID = 0x0100) on Headset
  if {$UsingPrimitives} {
    SDC_SERVICE_ATTRIBUTE_REQ - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x01 0x00} 10
    set result [SDC_SERVICE_ATTRIBUTE_CFM]
  } else {
    set result [sdc_service_attribute_req - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x01 0x00} 10]
  }
  puts "Service Attribute Result for ServiceName: $result"

  #Search for the Remote Audio Volume Control Attribute (Attribute ID = 0x0302) on Headset
  if {$UsingPrimitives} {
    SDC_SERVICE_ATTRIBUTE_REQ - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x03 0x02} 10
    set result [SDC_SERVICE_ATTRIBUTE_CFM]
  } else {
    set result [sdc_service_attribute_req - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} $srh 5 {0x35 0x3 0x09 0x03 0x02} 10]
  }
  puts "Service Attribute Result for Remote Audio Volume Control: $result"
}

# Search for Service Attributes on the Headset
proc ServiceSearchAttributes {} {
  global UsingPrimitives
  global bd_addr.lap bd_addr.uap bd_addr.nap

  #Get Service Record handle for the Headset
  set srh [SearchHeadsetServiceRecord]
  puts [format "Service Record Handle: 0x%x" $srh]

  #search for Headset service (UUID = 0x1108 record and ServiceName Attribute (Attribute ID = 0x0100) on Headset
  if {$UsingPrimitives} {
    SDC_SERVICE_SEARCH_ATTRIBUTE_REQ - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} 5 {0x35 0x3 0x19 0x11 0x08} 5 {0x35 0x3 0x09 0x01 0x00} 10
    set result [SDC_SERVICE_SEARCH_ATTRIBUTE_CFM]
  } else {
    set result [sdc_service_search_attribute_req - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} 5 {0x35 0x3 0x19 0x11 0x08} 5 {0x35 0x3 0x09 0x01 0x00} 10]
  }
  puts "Service Search Attribute Result for Headset and ServiceName: $result"
}

# Terminate a Search
proc TerminateSearch {} {
  global bd_addr.lap bd_addr.uap bd_addr.nap

  #Get Service Record handle for the Headset
  set srh [SearchHeadsetServiceRecord]
  puts [format "Service Record Handle: 0x%x" $srh]

  #search for Headset service (UUID = 0x1108 record and ServiceName Attribute (Attribute ID = 0x0100) on Headset
  #then terminate the search
  SDC_SERVICE_SEARCH_ATTRIBUTE_REQ - ${bd_addr.lap} ${bd_addr.uap} ${bd_addr.nap} 5 {0x35 0x3 0x19 0x11 0x08} 5 {0x35 0x3 0x09 0x01 0x00} 10
  SDC_TERMINATE_PRIMITIVE_REQ
}

# Unregister AG Service Record
proc UnRegisterAGServiceRecord {ph recordhandle} {
  global UsingPrimitives
  if {$UsingPrimitives} {
    SDS_UNREGISTER_REQ $ph $recordhandle
    SDS_UNREGISTER_CFM
  } else {
    sds_unregister_req $ph $recordhandle
  }
}

# Main Routine
proc mainroutine {} {
  global phandle ag_server_chan UsingPrimitives

  # Register AG Service Record
  set agrecordhandle [lindex [RegisterAGServiceRecord $ag_server_chan] 1]

  # Register with Security Manager
  DM_SM_REGISTER_REQ 1 $ag_server_chan 0 0x13 0

  # Pair with Headset
  PairWithHeadset

  # Request to Open SDC Search
  StartSDPSearch 0

  # Wait for Link Key Request and Reject It 
  WaitLinkKeyRequestReject

  # Wait for PIN Code and Respond with Stored Value = 1234
  WaitPinCodeRequest

  # Wait for Link Key
  WaitLinkKeyRequest

  # Wait for Open SDC Search Confirmed
  StartSDPSearch 1

  # Search for HID Service Record on the Headset
  SearchHIDServiceRecord

  # Search for Generic Audio Service on the Headset
  SearchGenericAudioServiceRecord

  # Search for Attributes on the Headset
  ServiceAttributes

  # Search for Service Attributes on the Headset
  ServiceSearchAttributes

  # Show Range of Attributes Available on the Headset
  sdc_range_search 0x1108

  # Terminate a Search
  TerminateSearch

  # Unregister AG Service Record
  UnRegisterAGServiceRecord $phandle $agrecordhandle

  # Close SDC Search
  if {$UsingPrimitives} {
    SDC_CLOSE_SEARCH_REQ
    set result [lindex [SDC_CLOSE_SEARCH_IND] 1]
  } else {
    set result [lindex [sdc_close_search_req] 1] 
  }
  switch $result {
    0x0 {puts "Connection disconnected by SDS Server"}
    0x1 {puts "Connection disconnected by SDS Client"}
  }
}
# Initialisation
BC_connect com2 bcsp 115200
#Set UsingPrimitives TRUE to use PRIMITIVES, FALSE to use library functions
set UsingPrimitives 1
set bd_addr.lap 0x10e47
set bd_addr.uap 0x5b
set bd_addr.nap 0x02
# turn off agpmode
BC_option -agpmode
# Register with the dm
dm_am_register_req
# Register with the rfcomm
rfc_register_req
# store the server_chan otherwise its overwritten by flow ctrl data inds
set ag_server_chan $server_chan
rfc_init_req
mainroutine
